@@ -1,76 +1,94 @@
-import time
 import copy
-import collections
+import time
 
 import numpy as np
-import networkx as nx
+import collections
 
+from .. import util
 from .. import caching
+from .. import exceptions
 from .. import transformations
 
+try:
+    import networkx as nx
+    _ForestParent = nx.DiGraph
+except BaseException as E:
+    # create a dummy module which will raise the ImportError
+    # or other exception only when someone tries to use networkx
+    nx = exceptions.ExceptionModule(E)
+    _ForestParent = object
+
 
 class TransformForest(object):
-
     def __init__(self, base_frame='world'):
+        # a graph structure, subclass of networkx DiGraph
         self.transforms = EnforcedForest()
+        # hashable, the base or root frame
         self.base_frame = base_frame
 
+        # save paths, keyed with tuple (from, to)
         self._paths = {}
-        self._updated = time.time()
+        # cache transformation matrices keyed with tuples
+        self._updated = str(np.random.random())
+        self._cache = caching.Cache(self.md5)
 
-        self._cache = caching.Cache(id_function=self.md5)
-
-    def update(self,
-               frame_to,
-               frame_from=None,
-               **kwargs):
+    def update(self, frame_to, frame_from=None, **kwargs):
         """
         Update a transform in the tree.
 
         Parameters
-        ---------
-        frame_from: hashable object, usually a string (eg 'world').
-                    If left as None it will be set to self.base_frame
-        frame_to:   hashable object, usually a string (eg 'mesh_0')
-
-        Additional kwargs (can be used in combinations)
-        ---------
-        matrix:      (4,4) array
-        quaternion:  (4) quaternion
-        axis:        (3) array
-        angle:       float, radians
-        translation: (3) array
-        geometry : Geometry object name
+        ------------
+        frame_from : hashable object
+          Usually a string (eg 'world').
+          If left as None it will be set to self.base_frame
+        frame_to :  hashable object
+          Usually a string (eg 'mesh_0')
+        matrix : (4,4) float
+          Homogeneous transformation matrix
+        quaternion :  (4,) float
+          Quaternion ordered [w, x, y, z]
+        axis : (3,) float
+          Axis of rotation
+        angle :  float
+          Angle of rotation, in radians
+        translation : (3,) float
+          Distance to translate
+        geometry : hashable
+          Geometry object name, e.g. 'mesh_0'
         """
+
+        self._updated = str(np.random.random())
+        self._cache.clear()
+
+        # if no frame specified, use base frame
         if frame_from is None:
             frame_from = self.base_frame
+        # convert various kwargs to a single matrix
         matrix = kwargs_to_matrix(**kwargs)
 
-        attr = {'matrix': matrix,
-                'time': time.time()}
-
+        # create the edge attributes
+        attr = {'matrix': matrix, 'time': time.time()}
+        # pass through geometry to edge attribute
         if 'geometry' in kwargs:
             attr['geometry'] = kwargs['geometry']
 
+        # add the edges
         changed = self.transforms.add_edge(frame_from,
                                            frame_to,
                                            **attr)
+        # set the node attribute with the geometry information
         if 'geometry' in kwargs:
-            nx.set_node_attributes(self.transforms,
-                                   name='geometry',
-                                   values={frame_to: kwargs['geometry']})
+            nx.set_node_attributes(
+                self.transforms,
+                name='geometry',
+                values={frame_to: kwargs['geometry']})
+        # if the edge update changed our structure
+        # dump our cache of shortest paths
         if changed:
             self._paths = {}
-        self._updated = time.time()
 
     def md5(self):
-        """
-        MD5 of transforms.
-
-        Currently only hashing update time.
-        """
-        result = str(int(self._updated * 1000)) + str(self.base_frame)
-        return result
+        return self._updated
 
     def copy(self):
         """
@@ -78,7 +96,7 @@ class TransformForest(object):
 
         Returns
         ------------
-        copied: TransformForest
+        copied : TransformForest
         """
         copied = TransformForest()
         copied.base_frame = copy.deepcopy(self.base_frame)
@@ -97,63 +115,105 @@ class TransformForest(object):
         for node in self.nodes:
             if node == base_frame:
                 continue
-            transform, geometry = self.get(frame_to=node,
-                                           frame_from=base_frame)
-            flat[node] = {'transform': transform.tolist(),
-                          'geometry': geometry}
+            transform, geometry = self.get(
+                frame_to=node, frame_from=base_frame)
+            flat[node] = {
+                'transform': transform.tolist(),
+                'geometry': geometry
+            }
         return flat
 
-    def to_gltf(self, mesh_index):
+    def to_gltf(self, scene):
         """
         Export a transforms as the 'nodes' section of a GLTF dict.
         Flattens tree.
 
         Returns
         --------
-        gltf: dict, with keys:
-                  'nodes': list of dicts
+        gltf : dict
+          with 'nodes' referencing a list of dicts
         """
-        gltf = collections.deque()
-        for node in self.nodes_geometry:
+
+        # geometry is an OrderedDict
+        # map mesh name to index: {geometry key : index}
+        mesh_index = {name: i for i, name
+                      in enumerate(scene.geometry.keys())}
+
+        # shortcut to graph
+        graph = self.transforms
+        # get the stored node data
+        node_data = dict(graph.nodes(data=True))
+
+        # list of dict, in gltf format
+        # start with base frame as first node index
+        result = [{'name': self.base_frame}]
+        # {node name : node index in gltf}
+        lookup = {self.base_frame: 0}
+
+        # collect the nodes in order
+        for node in node_data.keys():
             if node == self.base_frame:
                 continue
-            transform, geometry = self.get(frame_to=node,
-                                           frame_from=self.base_frame)
-            if geometry is None:
-                continue
-            gltf.append({'matrix': transform.T.reshape(-1).tolist(),
-                         'mesh': mesh_index[geometry],
-                         'name': node})
-        # we have flattened tree, so all nodes will be child of world
-        gltf.appendleft({'name': self.base_frame,
-                         'children': list(range(1, 1 + len(gltf)))})
-        result = {'nodes': list(gltf)}
+            lookup[node] = len(result)
+            result.append({'name': node})
 
-        return result
+        # then iterate through to collect data
+        for info in result:
+            # name of the scene node
+            node = info['name']
+            # store children as indexes
+            children = [lookup[k] for k in graph[node].keys()]
+            if len(children) > 0:
+                info['children'] = children
+            # if we have a mesh store by index
+            if 'geometry' in node_data[node]:
+                info['mesh'] = mesh_index[node_data[node]['geometry']]
+            # check to see if we have camera node
+            if node == scene.camera.name:
+                info['camera'] = 0
+            try:
+                # try to ignore KeyError and StopIteration
+                # parent-child transform is stored in child
+                parent = next(iter(graph.predecessors(node)))
+                # get the (4, 4) homogeneous transform
+                matrix = graph.get_edge_data(parent, node)['matrix']
+                # only include matrix if it is not an identity matrix
+                if np.abs(matrix - np.eye(4)).max() > 1e-5:
+                    info['matrix'] = matrix.T.reshape(-1).tolist()
+            except BaseException:
+                continue
+        return {'nodes': result}
 
     def to_edgelist(self):
         """
-        Export the current transforms as a list of edge tuples, with
-        each tuple having the format:
+        Export the current transforms as a list of
+        edge tuples, with each tuple having the format:
         (node_a, node_b, {metadata})
 
         Returns
-        -------
-        edgelist: (n,) list of tuples
+        ---------
+        edgelist : (n,) list
+          Of edge tuples
         """
         # save cleaned edges
         export = []
         # loop through (node, node, edge attributes)
         for edge in nx.to_edgelist(self.transforms):
-            a, b, c = edge
+            a, b, attr = edge
             # geometry is a node property but save it to the
             # edge so we don't need two dictionaries
-            if 'geometry' in self.transforms.node[b]:
-                c['geometry'] = self.transforms.node[b]['geometry']
+            try:
+                b_attr = self.transforms.nodes[b]
+            except BaseException:
+                # networkx 1.X API
+                b_attr = self.transforms.node[b]
+            # apply node geometry to edge attributes
+            if 'geometry' in b_attr:
+                attr['geometry'] = b_attr['geometry']
             # save the matrix as a float list
-            c['matrix'] = np.asanyarray(c['matrix'],
-                                        dtype=np.float64).tolist()
-            export.append((a, b, c))
+            attr['matrix'] = np.asanyarray(
+                attr['matrix'], dtype=np.float64).tolist()
+            export.append((a, b, attr))
         return export
 
     def from_edgelist(self, edges, strict=True):
@@ -200,7 +260,8 @@ class TransformForest(object):
 
         Returns
         -------------
-        nodes: (n,) array, of node names
+        nodes : (n,) array
+          All node names
         """
         nodes = np.array(list(self.transforms.nodes()))
         return nodes
@@ -212,17 +273,58 @@ class TransformForest(object):
 
         Returns
         ------------
-        nodes_geometry: (m,) array, of node names
+        nodes_geometry : (m,) array
+          Node names which have geometry associated
         """
-
-        nodes = np.array([n for n in self.transforms.nodes()
-                          if 'geometry' in self.transforms.node[n]])
+        nodes = np.array(
+            [n for n, attr in self.transforms.nodes(data=True)
+             if 'geometry' in attr])
 
         return nodes
 
-    def get(self,
-            frame_to,
-            frame_from=None):
+    @caching.cache_decorator
+    def geometry_nodes(self):
+        """
+        Which nodes have this geometry?
+
+        Returns
+        ------------
+        geometry_nodes : dict
+          Geometry name : (n,) node names
+        """
+        res = collections.defaultdict(list)
+        for node, attr in self.transforms.nodes(data=True):
+            if 'geometry' in attr:
+                res[attr['geometry']].append(node)
+        return res
+
+    def remove_geometries(self, geometries):
+        """
+        Remove the reference for specified geometries from nodes
+        without deleting the node.
+
+        Parameters
+        ------------
+        geometries : list or str
+          Name of scene.geometry to dereference.
+        """
+        # make sure we have a set of geometries to remove
+        if util.is_string(geometries):
+            geometries = [geometries]
+        geometries = set(geometries)
+
+        # remove the geometry reference from the node without deleting nodes
+        # this lets us keep our cached paths, and will not screw up children
+        for node, attrib in self.transforms.nodes(data=True):
+            if 'geometry' in attrib and attrib['geometry'] in geometries:
+                attrib.pop('geometry')
+
+        # it would be safer to just run _cache.clear
+        # but the only property using the geometry should be
+        # nodes_geometry: if this becomes not true change this to clear!
+        self._cache.cache.pop('nodes_geometry', None)
+
+    def get(self, frame_to, frame_from=None):
         """
         Get the transform from one frame to another, assuming they are connected
         in the transform tree.
@@ -230,38 +332,60 @@ class TransformForest(object):
         If the frames are not connected a NetworkXNoPath error will be raised.
 
         Parameters
-        ---------
-        frame_from: hashable object, usually a string (eg 'world').
-                    If left as None it will be set to self.base_frame
-        frame_to:   hashable object, usually a string (eg 'mesh_0')
+        ------------
+        frame_to : hashable
+          Node name, usually a string (eg 'mesh_0')
+        frame_from : hashable
+          Node name, usually a string (eg 'world').
+          If None it will be set to self.base_frame
 
         Returns
-        ---------
-        transform:  (4,4) homogenous transformation matrix
+        ----------
+        transform : (4, 4) float
+          Homogeneous transformation matrix
         """
 
         if frame_from is None:
             frame_from = self.base_frame
 
-        cache_key = str(frame_from) + ':' + str(frame_to)
+        # look up transform to see if we have it already
+        cache_key = (frame_from, frame_to)
         cached = self._cache[cache_key]
         if cached is not None:
             return cached
 
-        transform = np.eye(4)
+        # get the path in the graph
         path = self._get_path(frame_from, frame_to)
 
+        # collect transforms along the path
+        transforms = []
+
         for i in range(len(path) - 1):
+            # get the matrix and edge direction
             data, direction = self.transforms.get_edge_data_direction(
                 path[i], path[i + 1])
             matrix = data['matrix']
             if direction < 0:
                 matrix = np.linalg.inv(matrix)
-            transform = np.dot(transform, matrix)
+            transforms.append(matrix)
+        # do all dot products at the end
+        if len(transforms) == 0:
+            transform = np.eye(4)
+        elif len(transforms) == 1:
+            transform = np.asanyarray(transforms[0], dtype=np.float64)
+        else:
+            transform = util.multi_dot(transforms)
 
-        geometry = None
-        if 'geometry' in self.transforms.node[frame_to]:
-            geometry = self.transforms.node[frame_to]['geometry']
+        try:
+            attr = self.transforms.nodes[frame_to]
+        except BaseException:
+            # networkx 1.X API
+            attr = self.transforms.node[frame_to]
+
+        if 'geometry' in attr:
+            geometry = attr['geometry']
+        else:
+            geometry = None
 
         self._cache[cache_key] = (transform, geometry)
 
@@ -282,7 +406,12 @@ class TransformForest(object):
         return graph_to_svg(self.transforms)
 
     def __contains__(self, key):
-        return key in self.transforms.node
+        try:
+            return key in self.transforms.nodes
+        except BaseException:
+            # networkx 1.X API
+            util.log.warning('upgrade networkx to version 2.X!')
+            return key in self.transforms.nodes()
 
     def __getitem__(self, key):
         return self.get(key)
@@ -296,17 +425,16 @@ class TransformForest(object):
     def clear(self):
         self.transforms = EnforcedForest()
         self._paths = {}
-        self._updated = time.time()
+        self._updated = str(np.random.random())
+        self._cache.clear()
 
-    def _get_path(self,
-                  frame_from,
-                  frame_to):
+    def _get_path(self, frame_from, frame_to):
         """
         Find a path between two frames, either from cached paths or
         from the transform graph.
 
         Parameters
-        ---------
+        ------------
         frame_from: a frame key, usually a string
                     eg, 'world'
         frame_to:   a frame key, usually a string
@@ -317,23 +445,26 @@ class TransformForest(object):
         path: (n) list of frame keys
               eg, ['mesh_finger', 'mesh_hand', 'world']
         """
+        # store paths keyed as a tuple
         key = (frame_from, frame_to)
-        if not (key in self._paths):
-            path = self.transforms.shortest_path_undirected(frame_from,
-                                                            frame_to)
+        if key not in self._paths:
+            # get the actual shortest paths
+            path = self.transforms.shortest_path_undirected(
+                frame_from, frame_to)
+            # store path to avoid recomputing
             self._paths[key] = path
+            return path
         return self._paths[key]
 
 
-class EnforcedForest(nx.DiGraph):
+class EnforcedForest(_ForestParent):
     """
     A subclass of networkx.DiGraph that will raise an error if an
     edge is added which would make the DiGraph not a forest or tree.
     """
 
     def __init__(self, *args, **kwargs):
-        self.flags = {'strict': False,
-                      'assert_forest': False}
+        self.flags = {'strict': False, 'assert_forest': False}
 
         for k, v in self.flags.items():
             if k in kwargs:
@@ -343,7 +474,7 @@ class EnforcedForest(nx.DiGraph):
         super(self.__class__, self).__init__(*args, **kwargs)
         # keep a second parallel but undirected copy of the graph
         # all of the networkx methods for turning a directed graph
-        # into an undirected graph are quite slow, so we do minor bookkeeping
+        # into an undirected graph are quite slow so we do minor bookkeeping
         self._undirected = nx.Graph()
 
     def add_edge(self, u, v, *args, **kwargs):
@@ -352,7 +483,7 @@ class EnforcedForest(nx.DiGraph):
             if self.flags['strict']:
                 raise ValueError('Edge must be between two unique nodes!')
             return changed
-        if self._undirected.has_edge(u, v):
+        elif self._undirected.has_edge(u, v):
             self.remove_edges_from([[u, v], [v, u]])
         elif len(self.nodes()) > 0:
             try:
@@ -362,9 +493,7 @@ class EnforcedForest(nx.DiGraph):
                         'Multiple edge path exists between nodes!')
                 self.disconnect_path(path)
                 changed = True
-            except (nx.NetworkXError,
-                    nx.NetworkXNoPath,
-                    nx.NetworkXException):
+            except (nx.NetworkXError, nx.NetworkXNoPath, nx.NetworkXException):
                 pass
         self._undirected.add_edge(u, v)
         super(self.__class__, self).add_edge(u, v, *args, **kwargs)
@@ -396,7 +525,10 @@ class EnforcedForest(nx.DiGraph):
         self.remove_edges_from(ebunch)
 
     def shortest_path_undirected(self, u, v):
-        path = nx.shortest_path(self._undirected, u, v)
+        try:
+            path = nx.shortest_path(self._undirected, u, v)
+        except BaseException as E:
+            raise E
         return path
 
     def get_edge_data_direction(self, u, v):
@@ -410,21 +542,13 @@ class EnforcedForest(nx.DiGraph):
         return data, direction
 
 
-def path_to_edges(path):
-    """
-    Turn an (n) path into a (2(n-1)) set of edges
-    """
-    return np.column_stack((path, path)).reshape(-1)[1:-1].reshape((-1, 2))
-
-
 def kwargs_to_matrix(**kwargs):
     """
     Turn a set of keyword arguments into a transformation matrix.
     """
-    matrix = np.eye(4)
     if 'matrix' in kwargs:
         # a matrix takes precedence over other options
-        matrix = kwargs['matrix']
+        matrix = np.asanyarray(kwargs['matrix'], dtype=np.float64)
     elif 'quaternion' in kwargs:
         matrix = transformations.quaternion_matrix(kwargs['quaternion'])
     elif ('axis' in kwargs) and ('angle' in kwargs):
